@@ -25,11 +25,15 @@
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/time.h>	/* needed for Sun 4.1.2 */
 #ifdef HAVE_SETRLIMIT
 #include <sys/resource.h>
 #endif /* HAVE_SETRLIMIT */
 #include <sys/utsname.h>
 
+#ifdef HAVE_NET_SOCKET_H
+#include <net/socket.h>
+#endif
 #ifdef HAVE_GETHOSTBYNAME
 #include <netdb.h>
 #endif /* HAVE_GETHOSTBYNAME */
@@ -38,6 +42,7 @@
 #include <hesiod.h>
 #endif
 
+#include "getopt.h"
 #include "fetchmail.h"
 #include "tunable.h"
 #include "smtp.h"
@@ -64,6 +69,7 @@ flag check_only;	    /* if --probe was set */
 flag versioninfo;	    /* emit only version info */
 char *user;		    /* the name of the invoking user */
 char *home;		    /* invoking user's home directory */
+char *fmhome;		    /* fetchmail's home directory */
 char *program_name;	    /* the name to prefix error messages with */
 flag configdump;	    /* dump control blocks for configurator */
 const char *fetchmailhost;  /* either `localhost' or the host's FQDN */
@@ -124,6 +130,7 @@ int main(int argc, char **argv)
     netrc_entry *netrc_list;
     char *netrc_file, *tmpbuf;
     pid_t pid;
+    int lastsig = 0;
 
 #ifdef __FreeBSD__
     dropprivs();
@@ -151,8 +158,8 @@ int main(int argc, char **argv)
     }
 
 #define IDFILE_NAME	".fetchids"
-    run.idfile = (char *) xmalloc(strlen(home)+sizeof(IDFILE_NAME)+1);
-    strcpy(run.idfile, home);
+    run.idfile = (char *) xmalloc(strlen(fmhome)+sizeof(IDFILE_NAME)+2);
+    strcpy(run.idfile, fmhome);
     strcat(run.idfile, "/");
     strcat(run.idfile, IDFILE_NAME);
   
@@ -236,7 +243,12 @@ int main(int argc, char **argv)
     /* logging should be set up early in case we were restarted from exec */
     if (run.use_syslog)
     {
+#if defined(LOG_MAIL)
 	openlog(program_name, LOG_PID, LOG_MAIL);
+#else
+	/* Assume BSD4.2 openlog with two arguments */
+	openlog(program_name, LOG_PID);
+#endif
 	report_init(-1);
     }
     else
@@ -250,9 +262,11 @@ int main(int argc, char **argv)
 		sizeof(PID_DIR) + sizeof(FETCHMAIL_PIDFILE));
 	sprintf(tmpbuf, "%s/%s", PID_DIR, FETCHMAIL_PIDFILE);
     } else {
-	xalloca(tmpbuf, char *, strlen(home) + sizeof(FETCHMAIL_PIDFILE) + 2);
-	strcpy(tmpbuf, home);
-	strcat(tmpbuf, "/.");
+	xalloca(tmpbuf, char *, strlen(fmhome) + sizeof(FETCHMAIL_PIDFILE) + 2);
+	strcpy(tmpbuf, fmhome);
+	strcat(tmpbuf, "/");
+        if (fmhome == home)
+ 	   strcat(tmpbuf, ".");
 	strcat(tmpbuf, FETCHMAIL_PIDFILE);
     }
 #undef FETCHMAIL_PIDFILE
@@ -274,7 +288,7 @@ int main(int argc, char **argv)
 
 #define	NETRC_FILE	".netrc"
     /* parse the ~/.netrc file (if present) for future password lookups. */
-    xalloca(netrc_file, char *, strlen (home) + sizeof(NETRC_FILE) + 1);
+    xalloca(netrc_file, char *, strlen(home) + sizeof(NETRC_FILE) + 2);
     strcpy (netrc_file, home);
     strcat (netrc_file, "/");
     strcat (netrc_file, NETRC_FILE);
@@ -484,7 +498,7 @@ int main(int argc, char **argv)
 			strlen(ctl->server.pollname) + 1);
 		(void) sprintf(tmpbuf, password_prompt,
 			       ctl->remotename, ctl->server.pollname);
-		ctl->password = xstrdup((char *)getpassword(tmpbuf));
+		ctl->password = xstrdup((char *)fm_getpassword(tmpbuf));
 	    }
     }
 
@@ -620,8 +634,11 @@ int main(int argc, char **argv)
 		}
 
 #if (defined(linux) && !INET6_ENABLE) || defined(__FreeBSD__)
-		/* interface_approve() does its own error logging */
-		if (!interface_approve(&ctl->server))
+		/*
+		 * Don't do monitoring if we were woken by a signal.
+		 * Note that interface_approve() does its own error logging.
+		 */
+		if (!interface_approve(&ctl->server, !lastsig))
 		    continue;
 #endif /* (defined(linux) && !INET6_ENABLE) || defined(__FreeBSD__) */
 
@@ -697,8 +714,6 @@ int main(int argc, char **argv)
 	 */
 	if (run.poll_interval)
 	{
-	    int lastsig;
-
 	    /* 
 	     * Because passwords can expire, it may happen that *all*
 	     * hosts are now out of the loop due to authfail
@@ -725,11 +740,11 @@ int main(int argc, char **argv)
 
 	    /*
 	     * OK, now pause util it's time for the next poll cycle.
-	     * A TRUE return indicates we received a wakeup signal;
+	     * A nonzero return indicates we received a wakeup signal;
 	     * unwedge all servers in case the problem has been
 	     * manually repaired.
 	     */
-	    if ((lastsig = idle(run.poll_interval)))
+	    if ((lastsig = interruptible_idle(run.poll_interval)))
 	    {
 #ifdef SYS_SIGLIST_DECLARED
 		report(stdout, 
@@ -756,23 +771,29 @@ int main(int argc, char **argv)
     exit(successes ? PS_SUCCESS : querystatus);
 }
 
+static void list_merge(struct idlist **dstl, struct idlist **srcl, int force)
+{
+    /*
+     * If force is off, modify dstl fields only when they're empty (treat srcl
+     * as defaults).  If force is on, modify each dstl field whenever scrcl
+     * is nonempty (treat srcl as an override).  
+     */
+    if (force ? !!*srcl : !*dstl)
+    {
+	struct idlist *cpl = copy_str_list(*srcl);
+
+	append_str_list(dstl, &cpl);
+    }
+}
+
 static void optmerge(struct query *h2, struct query *h1, int force)
 /* merge two options records */
 {
-    /*
-     * If force is off, modify h2 fields only when they're empty (treat h1
-     * as defaults).  If force is on, modify each h2 field whenever h1
-     * is nonempty (treat h1 as an override).  
-     */
-#define LIST_MERGE(dstl, srcl) if (force ? !!srcl : !dstl) \
-    						free_str_list(&dstl), \
-						append_str_list(&dstl, &srcl)
-    LIST_MERGE(h2->server.localdomains, h1->server.localdomains);
-    LIST_MERGE(h2->localnames, h1->localnames);
-    LIST_MERGE(h2->mailboxes, h1->mailboxes);
-    LIST_MERGE(h2->smtphunt, h1->smtphunt);
-    LIST_MERGE(h2->antispam, h1->antispam);
-#undef LIST_MERGE
+    list_merge(&h2->server.localdomains, &h1->server.localdomains, force);
+    list_merge(&h2->localnames, &h1->localnames, force);
+    list_merge(&h2->mailboxes, &h1->mailboxes, force);
+    list_merge(&h2->smtphunt, &h1->smtphunt, force);
+    list_merge(&h2->antispam, &h1->antispam, force);
 
 #define FLAG_MERGE(fld) if (force ? !!h1->fld : !h2->fld) h2->fld = h1->fld
     FLAG_MERGE(server.via);
@@ -1694,11 +1715,11 @@ static void dump_params (struct runctl *runp,
 #endif
 
 	if (ctl->server.plugin)
-	    printf(_("  Server connections will be mode via plugin %s (--plugin %s).\n"), ctl->server.plugin, ctl->server.plugin);
+	    printf(_("  Server connections will be made via plugin %s (--plugin %s).\n"), ctl->server.plugin, ctl->server.plugin);
 	else if (outlevel >= O_VERBOSE)
 	    printf(_("  No plugin command specified.\n"));
 	if (ctl->server.plugout)
-	    printf(_("  Listener connections will be mode via plugout %s (--plugout %s).\n"), ctl->server.plugout, ctl->server.plugout);
+	    printf(_("  Listener connections will be made via plugout %s (--plugout %s).\n"), ctl->server.plugout, ctl->server.plugout);
 	else if (outlevel >= O_VERBOSE)
 	    printf(_("  No plugout command specified.\n"));
 
