@@ -351,6 +351,7 @@ extern int linelimit;		/* limit # lines retrieved per site */
 extern flag versioninfo;	/* emit only version info */
 extern char *user;		/* name of invoking user */
 extern char *home;		/* home directory of invoking user */
+extern char *fmhome;		/* fetchmail home directory */
 extern int pass;		/* number of re-polling pass */
 extern flag configdump;		/* dump control blocks as Python dictionary */
 extern const char *fetchmailhost;
@@ -401,16 +402,19 @@ extern int phase;
 #define STAGE_GETRANGE	1
 #define STAGE_GETSIZES	2
 #define STAGE_FETCH	3
-#define STAGE_LOGOUT	4
+#define STAGE_IDLE	4
+#define STAGE_LOGOUT	5
 extern int stage;
 
+extern int mytimeout;
+
 /* mark values for name lists */
 #define XMIT_ACCEPT	1	/* accepted; matches local domain or name */
 #define XMIT_REJECT	2	/* rejected; no match */
 #define XMIT_RCPTBAD	3	/* SMTP listener rejected the name */ 
 
 /* idle.c */
-int idle(int interval);
+int interruptible_idle(int interval);
 
 /* sink.c: forwarding */
 int stuffline(struct query *, char *);
@@ -433,6 +437,7 @@ char *nxtaddr(const char *);
 void initialize_saved_lists(struct query *, const char *);
 struct idlist *save_str(struct idlist **, const char *, flag);
 void free_str_list(struct idlist **);
+struct idlist *copy_str_list(struct idlist *idl);
 void save_str_pair(struct idlist **, const char *, const char *);
 void free_str_pair_list(struct idlist **);
 int delete_str(struct idlist **, int);
@@ -470,7 +475,7 @@ extern int  UnMimeBodyline(unsigned char **buf, flag delimited, flag issoftline)
 void interface_init(void);
 void interface_parse(char *, struct hostdata *);
 void interface_note_activity(struct hostdata *);
-int interface_approve(struct hostdata *);
+int interface_approve(struct hostdata *, flag domonitor);
 
 /* xmalloc.c */
 #if defined(HAVE_VOIDPOINTER)
@@ -503,7 +508,7 @@ int parsecmdline (int, char **, struct runctl *, struct query *);
 char *MD5Digest (unsigned char *);
 int POP3_auth_rpa(unsigned char *, unsigned char *, int socket);
 int daemonize(const char *, void (*)(int));
-char *getpassword(char *);
+char *fm_getpassword(char *);
 void escapes(const char *, char *);
 char *visbuf(const char *);
 const char *showproto(int);
