@@ -81,12 +81,11 @@ void interface_init(void)
     {
 	int major, minor;
 
-	if (fscanf(fp, "%d.%d.%*d", &major, &minor) != 2)
-	    return;
-
-	if (major >= 2 && minor >= 2)
+	if (fscanf(fp, "%d.%d.%*d", &major, &minor) >= 2
+					&& major >= 2 && minor >= 2)
 	    /* Linux 2.2 -- transmit packet count in 10th field */
 	    netdevfmt = "%d %d %*d %*d %*d %d %*d %*d %*d %d %*d %*d %d";
+	pclose(fp);
     }
 }
 
@@ -434,7 +433,7 @@ void interface_note_activity(struct hostdata *hp)
 #endif
 }
 
-int interface_approve(struct hostdata *hp)
+int interface_approve(struct hostdata *hp, flag domonitor)
 /* return TRUE if OK to poll, FALSE otherwise */
 {
 	ifinfo_t ifinfo;
@@ -460,7 +459,7 @@ int interface_approve(struct hostdata *hp)
 	}
 
 	/* if not monitoring link, all done */
-	if (!hp->monitor)
+	if (!domonitor || !hp->monitor)
 		return(TRUE);
 
 #ifdef	ACTIVITY_DEBUG
